import json
import os
import redis
import random
import queue
import threading
import logging

HOSTNAME = 'test-dashboard-storage02.geant.org'
INDEX = 0
NUM_THREADS = 20


def _redis_client_proc(key_queue, value_queue):
    r = redis.StrictRedis(host=HOSTNAME, db=INDEX)
    while True:
        key = key_queue.get()

        # contract is that None means no more requests
        if not key:
            break

        logging.debug(f'key: {key}')
        value_queue.put({
            'key': key,
            'value': r.get(key).decode('utf-8')
        })

    # contract is to return None when finished
    value_queue.put(None)


def docs():
    threads = []
    response_queue = queue.Queue()

    for _ in range(NUM_THREADS):
        q = queue.Queue()
        t = threading.Thread(
            target=_redis_client_proc,
            args=[q, response_queue])
        t.start()
        threads.append({'thread': t, 'queue': q})

    r = redis.StrictRedis(host=HOSTNAME, db=INDEX)
    # scan with bigger batches, to mitigate network latency effects
    for k in r.scan_iter(count=1000):
        t = random.choice(threads)
        t['queue'].put(k.decode('utf-8'))

    # tell all threads there are no more keys coming
    for t in threads:
        t['queue'].put(None)

    num_finished = 0
    # read values from response_queue until we receive
    # None len(threads) times
    while num_finished < len(threads):
        value = response_queue.get()
        if not value:
            num_finished += 1
            logging.info(
                'one thread finished '
                f'({len(threads) - num_finished} left)')
            continue
        yield value

    # cleanup like we're supposed to, even though it's python
    for t in threads:
        t['thread'].join(timeout=0.5)  # timeout, for sanity


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)

    d = {}
    for item in docs():
        d[item['key']] = item['value']

    data_filename = os.path.join(os.path.dirname(__file__), "router-info.json")

    with open(data_filename, "w") as f:
        f.write(json.dumps(d))
